@@ -403,20 +403,38 @@ class Model():
         model = Model(model_json["name"], model_json["tag"], model_json["function_name"], model_json["input"], model_json["category"])
         model.id = model_json.get('id', None)
         return model
+
+class MaterialParam():
+    def __init__(self, name : str, material : Material) -> None:
+        self.id = None
+        self.name = name
+        self.submitted_by = None
+        self.material = material
+
+    def to_json(self):
+        material_param_json = deepcopy(self.__dict__)
+        material_param_json["material"] = self.material.id
+        return material_param_json
+    
+    @classmethod
+    def load_json(cls, materialp_json : dict):
+        material_param = MaterialParam(materialp_json["name"], get_material(materialp_json["material"]))
+        material_param.id = materialp_json.get('id', None)
+        material_param.submitted_by = materialp_json.get('submitted_by', None)
+        return material_param
     
 class ModelParams():
-    def __init__(self, test : Test, model : Model, params : dict) -> None:
+    def __init__(self, model : Model, material_param : MaterialParam, params : dict) -> None:
         self.id = None
         self.submitted_by = None
-        self.user = None
-        self.test = test
+        self.material_param = material_param
         self.model = model
         self.params = params
 
     def to_json(self):
         modelp_json = deepcopy(self.__dict__)
-        modelp_json["test"] = self.test.id
         modelp_json["model"] = self.model.id
+        modelp_json["material_param"] = self.material_param.id
         return modelp_json
     
     def get_graph(self):
@@ -439,9 +457,9 @@ class ModelParams():
     
     @classmethod
     def load_json(cls, modelp_json : dict):
-        modelp = ModelParams(get_test(modelp_json["test"]), get_model(modelp_json["model"]), modelp_json["params"])
+        modelp = ModelParams(get_test(modelp_json["test"]), get_model(modelp_json["model"]), get_material_param(modelp_json["material_param"]), modelp_json["params"])
         modelp.id = modelp_json.get('id', None)
-        modelp.user = modelp_json.get('user', None)
+        # modelp.user = modelp_json.get('user', None)
         modelp.submitted_by = modelp_json.get('submitted_by', None)
         return modelp
 
@@ -1040,6 +1058,52 @@ def register_model_params(login_token : str, modelp : ModelParams):
         modelp.submitted_by = response.json()["submitted_by"]
         return modelp
 
+# Material Params
+
+def get_material_param(materialp_id : int):
+    """Retrieve material parameters by id.
+    
+    Parameters
+    ----------
+    materialp_id : int
+        The id of the material parameters to be fetched
+    """
+
+    url = f"{URL}/materialparams/{materialp_id}/"
+
+    response = requests.get(url)
+
+    if response.status_code != 200:
+        raise APIFailedRequest(response)
+    
+    materialp_data = response.json()
+
+    return MaterialParam.load_json(materialp_data)
+
+def register_material_param(login_token : str, materialp : MaterialParam):
+    """Save material parameters to the database.
+    
+    Parameters
+    ----------
+    login_token : str
+        The log-in token that can be retrieved from the authenticate function
+    materialp : MaterialParam
+        The material parameter to be saved 
+    """
+    headers = {"Authorization": f"Token {login_token}"}
+    
+    url = f"{URL}/materialparams/"
+    
+    body = materialp.to_json()
+    response = requests.post(url, json=body, headers=headers)
+
+    if response.status_code != 201:
+        raise APIFailedRequest(response)
+    else:
+        materialp.id = response.json()["id"]
+        materialp.submitted_by = response.json()["submitted_by"]
+        print(f"register_material_param: Successfully registered material parameter {materialp.name} with id {materialp.id}.")
+        return materialp
 
 # Institution
 
